// -*- C++ -*-                                                                 |
//-----------------------------------------------------------------------------+
//                                                                             |
// bsoul.h                                                                     |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2002                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fuer Astronomie,                              |
//           Koenigstuhl 17, D-69117 Heidelberg, Germany                       |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
// class basic_soul           fully inline                                     |
//                                                                             |
//-----------------------------------------------------------------------------+
#ifndef included_bsoul_h
#define included_bsoul_h

#ifndef included_flag_h
#  include <nbdy/flag.h>
#endif

#ifndef included_iomanip
#  include <iomanip>
#  define included_iomanip
#endif

using std::ostream;
using std::setw;
using std::setprecision;

////////////////////////////////////////////////////////////////////////////////
namespace nbdy {
  //////////////////////////////////////////////////////////////////////////////
  // class nbdy::basic_soul<BODY_SOURCES>                                       
  //////////////////////////////////////////////////////////////////////////////
  template<typename BODY_SOURCES>
  class basic_soul : public basic_flag {
  private:
    basic_soul           (const basic_soul&);      // not implemented           
    basic_soul& operator=(const basic_soul&);      // not implemented           
    //--------------------------------------------------------------------------
    // basic types                                                              
    //--------------------------------------------------------------------------
  public:
    typedef real         scalar_type;              // type of real numbers      
    typedef vect         vector_type;              // type of vector of reals   
    typedef BODY_SOURCES sources_type;             // type of body_sources      
    //--------------------------------------------------------------------------
    // data of class basic_soul                                                 
    //--------------------------------------------------------------------------
  private:
    vector_type COFM;                              // center of mass = position 
    unsigned    LINK;                              // associated body/index     
    //--------------------------------------------------------------------------
    // constructor                                                              
    //--------------------------------------------------------------------------
  protected:
    basic_soul() {}
    //--------------------------------------------------------------------------
    // non-const methods                                                        
    //--------------------------------------------------------------------------
    void         link(unsigned  const&L) { LINK = L; }
    vector_type &cofm()                  { return COFM; }
    scalar_type &cofm(const int i)       { return COFM[i]; }
    //--------------------------------------------------------------------------
  public:
    void copy_link (const basic_soul*S)  { LINK = S->LINK; }
    void copy_basic(const basic_soul*S)  { COFM = S->COFM; LINK = S->LINK; }
    //--------------------------------------------------------------------------
    // data access                                                              
    //--------------------------------------------------------------------------
    vector_type    const&cofm  () const { return COFM; }
    unsigned       const&mybody() const { return LINK; }
    //--------------------------------------------------------------------------
    // flag manipulations to use with bodies                                    
    //--------------------------------------------------------------------------
    void copy_body_flag(const BODY_SOURCES*B) {
      set_part_flag(&(B->flag(mybody())),basic_flag::BODY_FLAGS);
    }
    //--------------------------------------------------------------------------
    void add_body_flag(const BODY_SOURCES*B) {
      add_part_flag(&(B->flag(mybody())),basic_flag::BODY_FLAGS);
    }
    //--------------------------------------------------------------------------
    void copy_sink_flag(const BODY_SOURCES*B) {
      set_part_flag(&(B->flag(mybody())),basic_flag::SINK);
    }
    //--------------------------------------------------------------------------
    // flag manipulations to use with arrays                                    
    //--------------------------------------------------------------------------
    void copy_body_flag(const flag_type*F) {
      set_part_flag(F[mybody()],basic_flag::BODY_FLAGS);
    }
    //--------------------------------------------------------------------------
    void add_body_flag(const flag_type*F) {
      add_part_flag(F[mybody()],basic_flag::BODY_FLAGS);
    }
    //--------------------------------------------------------------------------
    void copy_sink_flag(const flag_type*F) {
      set_part_flag(F[mybody()],basic_flag::SINK);
    }
    //--------------------------------------------------------------------------
    // other non-const methods                                                  
    //--------------------------------------------------------------------------
    void copy(const basic_soul*S)  {
      copy_basic(S);
      set_part_flag(S,basic_flag::BODY_FLAGS);
    }
    //--------------------------------------------------------------------------
    // dump soul basic data                                                     
    //--------------------------------------------------------------------------
    static void dump_head(ostream&o) {
      o<<" flg mybody          position";
    }
    //--------------------------------------------------------------------------
    void dump(ostream&o) const
    {
      o<<" "<<setw(3)<<flag_type(*this)
       <<" "<<setw(6)<<LINK;
      for(register int i=0; i!=NDIM; ++i)
	o<<" "<<setw(8)<<setprecision(4)<<cofm()[i];
    }
  };
}
////////////////////////////////////////////////////////////////////////////////
#endif  // included_bsoul_h
