// -*- C++ -*-                                                                 |
//-----------------------------------------------------------------------------+
//                                                                             |
// drct.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2002                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fuer Astronomie,                              |
//           Koenigstuhl 17, D-69117 Heidelberg, Germany                       |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// Defines the constant array DIRECT, which are the default control parameters |
// for direct summation as follows (see also Appendix B of Dehnen 2002).       |
//                                                                             |
// DIRECT[0] = N_cb^pre:  a C-B interaction is done via direct summation       |
//                        if N_cell <= N_cb^pre                                |
//                                                                             |
// DIRECT[1] = N_cb^post: a not well-separated C-B interaction is done via     |
//                        direct summation if N_cell <= N_cb^post              |
//                                                                             |
// DIRECT[2] = N_cc^post: a not well-separated C-C interaction is done via     |
//                        direct summation if N_cell_1,N_cell_2 <= N_cc^post   |
//                                                                             |
// DIRECT[3] = N_cs:      a cell self-interation is done via direct summation  |
//                        if N_cell <= N_cs                                    |
//                                                                             |
//-----------------------------------------------------------------------------+
#ifndef included_drct_h
#define included_drct_h
////////////////////////////////////////////////////////////////////////////////
namespace nbdy {
#if NDIM==2
  const unsigned DIRECT[4] = {3, 64, 8,12};
#else
  const unsigned DIRECT[4] = {3,128,6,64};             
#endif
}
////////////////////////////////////////////////////////////////////////////////
#endif // included_drct_h
