// -*- C++ -*-                                                                 |
//-----------------------------------------------------------------------------+
//                                                                             |
// flag.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fuer Astronomie,                              |
//           Koenigstuhl 17, D-69117 Heidelberg, Germany                       |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
// class basic_flag                     fully inline                           |
//                                                                             |
//-----------------------------------------------------------------------------+
#ifndef included_flag_h
#define included_flag_h 1

#ifndef included_auxx_h
#include <nbdy/auxx.h>
#endif

////////////////////////////////////////////////////////////////////////////////
namespace nbdy {
  //////////////////////////////////////////////////////////////////////////////
  // class nbdy::basic_flag                                                     
  //////////////////////////////////////////////////////////////////////////////
  class basic_flag {
    //--------------------------------------------------------------------------
    // type of class basic_flag                                                 
    //--------------------------------------------------------------------------
  public:
    enum {
      SINK          =   1,                      // tree node is sink            
      NOT_IN_TREE   =   2,                      // body ignored by tree         
      SPH           =   4,                      // tree node has sph bodies     
      STICKY        =   8,                      // tree node has sticky bodies  
      BODY_FLAGS    =  15,                      // sum of all body flags        
      AL_SINK       =  16                       // all children are sinks       
    };
    //--------------------------------------------------------------------------
    // data of class basic_flag                                                 
    //--------------------------------------------------------------------------
  private:
    flag_type FLAG;                             // our flag                     
    //--------------------------------------------------------------------------
    // constructors                                                             
    //--------------------------------------------------------------------------
  public:
    basic_flag           ()                   : FLAG(0) {}
    basic_flag           (const flag_type&F)  : FLAG(F) {}
    basic_flag           (const basic_flag&F) : FLAG(F.FLAG) {}
    basic_flag& operator=(const basic_flag&F) { FLAG = F.FLAG; return *this; }
    //--------------------------------------------------------------------------
    // non const methods                                                        
    //--------------------------------------------------------------------------
    void reset_flag   ()                                    { FLAG = 0; }
    void set_flag     (const flag_type&F)                   { FLAG = F; }
    void set_part_flag(const flag_type&F, const flag_type&P){ FLAG = F&P; }
    void add_flag     (const flag_type&F)                   { FLAG|= F; }
    void add_part_flag(const flag_type&F, const flag_type&P){ FLAG|= F&P; }
    void un_flag      (const flag_type&F)                   { FLAG&= ~F; }
    //--------------------------------------------------------------------------
    void set_flag     (const basic_flag*F)                  { FLAG = F->FLAG; }
    void set_part_flag(const basic_flag*F,const flag_type&P){ FLAG = F->FLAG&P;}
    void add_flag     (const basic_flag*F)                  { FLAG|= F->FLAG; }
    void add_part_flag(const basic_flag*F,const flag_type&P){ FLAG|= F->FLAG&P;}
    void un_flag      (const basic_flag*F)                  { FLAG&= ~F->FLAG; }
    //--------------------------------------------------------------------------
    // boolean information via members                                          
    //--------------------------------------------------------------------------
    bool is_set(const flag_type &T) const { return FLAG & T; }
    //--------------------------------------------------------------------------
    // boolean information via friends                                          
    //--------------------------------------------------------------------------
    friend bool is_sink   (const basic_flag*F) { return F->is_set(SINK); }
    friend bool al_sink   (const basic_flag*F) { return F->is_set(AL_SINK); }
    friend bool is_in_tree(const basic_flag*F) { return!F->is_set(NOT_IN_TREE); }
    friend bool is_sph    (const basic_flag*F) { return F->is_set(SPH); }
    friend bool is_sticky (const basic_flag*F) { return F->is_set(STICKY); }
    //--------------------------------------------------------------------------
    friend bool is_sink   (const basic_flag&F) { return F.is_set(SINK); }
    friend bool al_sink   (const basic_flag&F) { return F.is_set(AL_SINK); }
    friend bool is_in_tree(const basic_flag&F) { return!F.is_set(NOT_IN_TREE); }
    friend bool is_sph    (const basic_flag&F) { return F.is_set(SPH); }
    friend bool is_sticky (const basic_flag&F) { return F.is_set(STICKY); }
    //--------------------------------------------------------------------------
    friend bool is_sink   (const flag_type&F) { return F & SINK; }
    friend bool al_sink   (const flag_type&F) { return F & AL_SINK; }
    friend bool is_in_tree(const flag_type&F) { return!(F & NOT_IN_TREE); }
    friend bool is_sph    (const flag_type&F) { return F & SPH; }
    friend bool is_sticky (const flag_type&F) { return F & STICKY; }
    //--------------------------------------------------------------------------
    // const conversion to const flag_type                                      
    //--------------------------------------------------------------------------
  protected:
    operator const flag_type () const { return FLAG; }
    //--------------------------------------------------------------------------
    // output                                                                   
    //--------------------------------------------------------------------------
    friend std::ostream& operator<< (std::ostream& o, const basic_flag& F)
    { return o<<F.FLAG; }
  };
}
////////////////////////////////////////////////////////////////////////////////
#endif // included_flag_h
