// -*- C++ -*-                                                                 |
//-----------------------------------------------------------------------------+
//                                                                             |
// grat.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2002                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fuer Astronomie,                              |
//           Koenigstuhl 17, D-69117 Heidelberg, Germany                       |
//                                                                             |
//-----------------------------------------------------------------------------+
#ifndef included_grat_h
#define included_grat_h

#ifndef included_cell_h
#  include <nbdy/cell.h>
#endif

#ifndef included_tree_h
#  include <nbdy/tree.h>
#endif
////////////////////////////////////////////////////////////////////////////////
namespace nbdy {
  //////////////////////////////////////////////////////////////////////////////
  //                                                                            
  // class nbdy::gravity_tree                                                   
  //                                                                            
  //////////////////////////////////////////////////////////////////////////////
  class gravity_tree : public tree<cell> {
    //--------------------------------------------------------------------------
    // data:                                                                    
    //--------------------------------------------------------------------------
  private:
    const areal *M;                                // array with masses         
    uint         P;                                // expansion order           
    int          nP,nS;                            // memory / cell             
    scalar_type *POLES;                            // memory for multipoles     
    scalar_type *STUFF;                            // memory for cell stuff     
    //--------------------------------------------------------------------------
    // private methods:                                                         
    //--------------------------------------------------------------------------
  private:
    //--------------------------------------------------------------------------
    // public methods:                                                          
    //--------------------------------------------------------------------------
  public:
    //--------------------------------------------------------------------------
    // construction:                                                            
    // - sets up the oct tree structure                                         
    // - recursively computes for every cell: mass, cofm, rmax                  
    //--------------------------------------------------------------------------
    //   construction from list of bodies -> build new tree from scratch        
    //--------------------------------------------------------------------------
    gravity_tree(const bodies_type*,               // I: bodies                 
		 const bool       = false,         // [I: allow for cell's eps] 
		 const vect     * = 0,             // [I: root center]          
		 const uint       = 6,             // [I: N_crit]               
		 const uint       = 50);           // [I: max tree depth]       
    //--------------------------------------------------------------------------
    //   construction from arrays of areal for positions                        
    //--------------------------------------------------------------------------
    gravity_tree(const flag_type*,                 // I: array with flags       
		 const areal    *[NDIM],           // I: arrays with x,y,z      
		 const areal    *,                 // I: array  with m          
		 const uint      ,                 // I: size of arrays         
		 const bool       = false,         // [I: allow for cell's eps] 
		 const vect     * = 0,             // [I: root center]          
		 const uint       = 6,             // [I: N_crit]               
		 const uint       = 50);           // [I: max tree depth]       
    //--------------------------------------------------------------------------
    // destruction                                                              
    //--------------------------------------------------------------------------
    ~gravity_tree() {
      if(POLES) delete[] POLES;
      if(STUFF) delete[] STUFF;
    }
    //--------------------------------------------------------------------------
    // re_use:                                                                  
    // - recursively computes for every cell: mass, cofm, rmax                  
    //--------------------------------------------------------------------------
    void reuse() const;
    //--------------------------------------------------------------------------
    // pre-compute the cells multipole moments                                  
    // - if the cells have no memory for multipoles yet, we give it to them     
    // - recursively compute the multipoles                                     
    //--------------------------------------------------------------------------
    void set_poles(const unsigned =3);             // [I: expansion order]      
    //--------------------------------------------------------------------------
  };
}
////////////////////////////////////////////////////////////////////////////////
#endif // included_grat_h
